#include <DS1302.h>
#include <Wire.h>
#include <Adafruit_PWMServoDriver.h>

Adafruit_PWMServoDriver board1 = Adafruit_PWMServoDriver(0x40);
Adafruit_PWMServoDriver board2 = Adafruit_PWMServoDriver(0x41);
Adafruit_PWMServoDriver board3 = Adafruit_PWMServoDriver(0x42);

#define SERVOMIN  544 // This is the 'minimum' pulse length count (out of 4096)
#define SERVOMAX  2400 // This is the 'maximum' pulse length count (out of 4096)
#define SERVO_FREQ 50 // Analog servos run at ~50 Hz updates

int hourPrevious;
int hourCurrent;
int minutePrevious;
int minuteCurrent;

int returndig1Previous;
int returndig1Current;
int returndig2;
int returndig3Previous;
int returndig3Current;
int returndig4;

const int RSTpin = 4;  //RST
const int DATpin = 3;  //DAT
const int CLKpin = 2;  //CLK

DS1302 rtc(RSTpin, DATpin, CLKpin); // Create RTC object.

///////////////////////////////////////////////////////////////////////////////////
void setup() {  
Serial.begin(9600); // initialize Serial connection

board1.begin();  // Initialize servo ccontrol boards
board2.begin();  
board3.begin();  
board1.setOscillatorFrequency(27000000);
board2.setOscillatorFrequency(27000000);
board3.setOscillatorFrequency(27000000);
board1.setPWMFreq(SERVO_FREQ);  // Analog servos run at ~50 Hz updates
board2.setPWMFreq(SERVO_FREQ); 
board3.setPWMFreq(SERVO_FREQ);

yield();

initialize();

}
///////////////////////////////////////////////////////////////////////////////////
void loop() {  // put your main code here, to run repeatedly:

Time t = rtc.time();                             //Get time from RTC

hourCurrent = (t.hr);                            // the current hour (0-23)
minuteCurrent = (t.min);                         // the current minute (0-59)

if (hourCurrent != hourPrevious) {                 // check to see if hour has changed
  returndig1Current = hourCurrent / 10;          // get 1st digit of hour
  returndig2 = hourCurrent % 10;                 // get 2nd digit of hour
  if (returndig1Current != returndig1Previous) {   // check to see if first digit has changed (2nd digit will always change)
    digit1(returndig1Current);                   // change digit 1 on clock to new number
    returndig1Previous = returndig1Current;      // store new number
  }
  digit2(returndig2);                            // change digit 2 on clock to new number
  hourPrevious = hourCurrent;                    // store new number
}

if (minuteCurrent != minutePrevious) {             // check to see if minute has changed
  returndig3Current = minuteCurrent / 10;        // get 1st digit of minute
  returndig4 = minuteCurrent % 10;               // get 2nd digit of minute
  if (returndig3Current != returndig3Previous) {   // check to see if first digit has changed (2nd digit will always change)
    digit3(returndig3Current);                   // change digit 3 on clock to new number
    returndig3Previous = returndig3Current;      // store new number
  }
  digit4(returndig4);                     // change digit 4 on clock to new number
  minutePrevious = minuteCurrent;                // store new number
}

delay(15000);                                     // wait 15 seconds to check if time has changed

}

///////////////////////////////////////////////////////////////////////////////////
int angleToPulse(int ang){
   int pulse1 = map(ang,0, 180, SERVOMIN,SERVOMAX); // map angle of 0 to 180 to Servo min and Servo max 
   int pulse2 = int(float(pulse1) / 1000000 * SERVO_FREQ * 4096);
//   Serial.print("Angle: ");Serial.print(ang);
//   Serial.print(" pulse: ");Serial.println(pulse);
   return pulse2;
}

///////////////////////////////////////////////////////////////////////////////////
void initialize() {
Time t = rtc.time();                             //Get time from RTC

hourCurrent = (t.hr);                            // the current hour (0-23)
minuteCurrent = (t.min);                         // the current minute (0-59)
//Serial.print(hourCurrent);Serial.print(":");Serial.println(minuteCurrent);
returndig1Current = hourCurrent / 10;            // get 1st digit of hour
returndig2 = hourCurrent % 10;                   // get 2nd digit of hour
returndig3Current = minuteCurrent / 10;          // get 1st digit of minute
returndig4 = minuteCurrent % 10;                 // get 2nd digit of minute
//Serial.print(returndig1Current);Serial.print(returndig2);Serial.print(returndig3Current);Serial.println(returndig4);

for (int dig1 = 0; dig1 <= 2; dig1++){                   // cycle 1st digit from 0-2
  digit1(dig1);
  //Serial.println(dig1);
  delay(100);
}
for (int dig1 = 0; dig1 <= returndig1Current; dig1++){   // cycle 1st digit from 0 to current time 
  digit1(dig1);
  //Serial.println(dig1);
  delay(100);
}

for (int dig2 = 0; dig2 <= 9; dig2++){                   // cycle 2nd digit from 0-9
  digit2(dig2);
  //Serial.println(dig2);
  delay(100);
}
for (int dig2 = 0; dig2 <= returndig2; dig2++){          // cycle 1st digit from 0 to current time
  digit2(dig2);
  //Serial.println(dig2);
  delay(100);
}

for (int dig3 = 0; dig3 <= 5; dig3++){                   // cycle 3rd digit from 0-5
  digit3(dig3);
  delay(100);
}
for (int dig3 = 0; dig3 <= returndig3Current; dig3++){   // cycle 1st digit from 0 to current time
  digit3(dig3);
  delay(100);
}

for (int dig4 = 0; dig4 <= 9; dig4++){                   // cycle 4th digit from 0-9
  digit4(dig4);
  delay(100);
}
for (int dig4 = 0; dig4 <= returndig4; dig4++){          // cycle 1st digit from 0 to current time
  digit4(dig4);
  delay(100);
}
}

/////////////////////{{{{ COPY AND PASTE CODE FROM EXCEL BELOW THIS LINE }}}}///////////////////////
void digit1(int number){
if (number == 0) {
board1.setPWM(4, 0, angleToPulse(30));delay(200);
board1.setPWM(5, 0, angleToPulse(75));delay(200);
board1.setPWM(2, 0, angleToPulse(29));delay(200);
board1.setPWM(3, 0, angleToPulse(100));delay(200);
board1.setPWM(8, 0, angleToPulse(38));delay(200);
board1.setPWM(7, 0, angleToPulse(128));delay(200);

}
else if (number == 1) {
board1.setPWM(5, 0, angleToPulse(25));delay(200);
board1.setPWM(4, 0, angleToPulse(20));delay(200);
board1.setPWM(3, 0, angleToPulse(21));delay(200);
board1.setPWM(2, 0, angleToPulse(18));delay(200);
board1.setPWM(1, 0, angleToPulse(25));delay(200);
board1.setPWM(0, 0, angleToPulse(24));delay(200);
board1.setPWM(10, 0, angleToPulse(67));delay(200);
board1.setPWM(6, 0, angleToPulse(125));delay(200);
}
else if (number == 2) {
board1.setPWM(5, 0, angleToPulse(69));delay(200);
board1.setPWM(4, 0, angleToPulse(67));delay(200);
board1.setPWM(3, 0, angleToPulse(63));delay(200);
board1.setPWM(2, 0, angleToPulse(31));delay(200);
board1.setPWM(1, 0, angleToPulse(104));delay(200);
board1.setPWM(0, 0, angleToPulse(72));delay(200);
board1.setPWM(10, 0, angleToPulse(32));delay(200);
board1.setPWM(8, 0, angleToPulse(3));delay(200);
board1.setPWM(7, 0, angleToPulse(20));delay(200);
board1.setPWM(6, 0, angleToPulse(21));delay(200);
}
}

void digit2(int number){
if (number == 0) {
board1.setPWM(14, 0, angleToPulse(48));delay(200);
board1.setPWM(13, 0, angleToPulse(129));delay(200);
}
else if (number == 1) {
board2.setPWM(3, 0, angleToPulse(77));delay(200);
board2.setPWM(1, 0, angleToPulse(28));delay(200);
board2.setPWM(0, 0, angleToPulse(30));delay(200);
board1.setPWM(15, 0, angleToPulse(34));delay(200);
board1.setPWM(14, 0, angleToPulse(31));delay(200);
board1.setPWM(13, 0, angleToPulse(41));delay(200);
board1.setPWM(12, 0, angleToPulse(29));delay(200);
board2.setPWM(6, 0, angleToPulse(53));delay(200);
board2.setPWM(2, 0, angleToPulse(132));delay(200);
board2.setPWM(3, 0, angleToPulse(117));delay(200);
}
else if (number == 2) {
board2.setPWM(1, 0, angleToPulse(89));delay(200);
board2.setPWM(0, 0, angleToPulse(88));delay(200);
board1.setPWM(15, 0, angleToPulse(88));delay(200);
board1.setPWM(14, 0, angleToPulse(51));delay(200);
board1.setPWM(13, 0, angleToPulse(127));delay(200);
board1.setPWM(12, 0, angleToPulse(85));delay(200);
board2.setPWM(6, 0, angleToPulse(17));delay(200);
board2.setPWM(4, 0, angleToPulse(14));delay(200);
board2.setPWM(3, 0, angleToPulse(10));delay(200);
board2.setPWM(2, 0, angleToPulse(24));delay(200);
}
else if (number == 3) {
board2.setPWM(2, 0, angleToPulse(61));delay(200);
board1.setPWM(14, 0, angleToPulse(86));delay(200);
board1.setPWM(13, 0, angleToPulse(93));delay(200);
board2.setPWM(3, 0, angleToPulse(117));delay(200);
board2.setPWM(2, 0, angleToPulse(23));delay(200);
}
else if (number == 4) {
board1.setPWM(14, 0, angleToPulse(48));delay(200);
board2.setPWM(3, 0, angleToPulse(80));delay(200);
board2.setPWM(1, 0, angleToPulse(48));delay(200);
board2.setPWM(0, 0, angleToPulse(49));delay(200);
board1.setPWM(15, 0, angleToPulse(122));delay(200);
board1.setPWM(13, 0, angleToPulse(41));delay(200);
board1.setPWM(12, 0, angleToPulse(30));delay(200);
board2.setPWM(6, 0, angleToPulse(53));delay(200);
board2.setPWM(4, 0, angleToPulse(50));delay(200);
board2.setPWM(2, 0, angleToPulse(131));delay(200);
board2.setPWM(3, 0, angleToPulse(117));delay(200);
board1.setPWM(14, 0, angleToPulse(86));delay(200);
}
else if (number == 5) {
board2.setPWM(1, 0, angleToPulse(91));delay(200);
board1.setPWM(13, 0, angleToPulse(94));delay(200);
board1.setPWM(12, 0, angleToPulse(85));delay(200);
board2.setPWM(7, 0, angleToPulse(12));delay(200);
board2.setPWM(6, 0, angleToPulse(16));delay(200);
board2.setPWM(5, 0, angleToPulse(53));delay(200);
board2.setPWM(4, 0, angleToPulse(15));delay(200);
board2.setPWM(2, 0, angleToPulse(22));delay(200);
}
else if (number == 6) {
board1.setPWM(14, 0, angleToPulse(48));delay(200);
board2.setPWM(3, 0, angleToPulse(116));delay(200);
}
else if (number == 7) {
 board2.setPWM(3, 0, angleToPulse(80));delay(200);
board2.setPWM(0, 0, angleToPulse(88));delay(200);
board1.setPWM(15, 0, angleToPulse(34));delay(200);
board1.setPWM(14, 0, angleToPulse(31));delay(200);
board1.setPWM(13, 0, angleToPulse(41));delay(200);
board1.setPWM(12, 0, angleToPulse(30));delay(200);
board2.setPWM(7, 0, angleToPulse(49));delay(200);
board2.setPWM(5, 0, angleToPulse(133));delay(200);
board2.setPWM(4, 0, angleToPulse(50));delay(200);
board2.setPWM(2, 0, angleToPulse(131));delay(200);
board2.setPWM(3, 0, angleToPulse(117));delay(200);
}
else if (number == 8) {
board2.setPWM(0, 0, angleToPulse(48));delay(200);
board1.setPWM(15, 0, angleToPulse(123));delay(200);
board1.setPWM(14, 0, angleToPulse(87));delay(200);
board1.setPWM(13, 0, angleToPulse(128));delay(200);
board1.setPWM(12, 0, angleToPulse(85));delay(200);
board2.setPWM(4, 0, angleToPulse(14));delay(200);
board2.setPWM(2, 0, angleToPulse(22));delay(200);
}
else if (number == 9) {
board1.setPWM(14, 0, angleToPulse(86));delay(200);
board1.setPWM(13, 0, angleToPulse(93));delay(200);
board2.setPWM(4, 0, angleToPulse(50));delay(200);
}
}

////////////////////////////////////////////////////////////{{{   DIGIT 3   }}}}
void digit3(int number){
if (number == 0) {
board2.setPWM(11, 0, angleToPulse(77));delay(200);
board2.setPWM(10, 0, angleToPulse(51));delay(200);
board2.setPWM(9, 0, angleToPulse(123));delay(200);
board2.setPWM(15, 0, angleToPulse(37));delay(200);
board3.setPWM(1, 0, angleToPulse(109));delay(200);
board3.setPWM(0, 0, angleToPulse(56));delay(200);
board2.setPWM(11, 0, angleToPulse(119));delay(200);
}
else if (number == 1) {
board3.setPWM(3, 0, angleToPulse(74));delay(200);
board2.setPWM(13, 0, angleToPulse(19));delay(200);
board2.setPWM(12, 0, angleToPulse(22));delay(200);
board2.setPWM(11, 0, angleToPulse(30));delay(200);
board2.setPWM(10, 0, angleToPulse(35));delay(200);
board2.setPWM(9, 0, angleToPulse(32));delay(200);
board2.setPWM(8, 0, angleToPulse(24));delay(200);
board2.setPWM(14, 0, angleToPulse(41));delay(200);
board3.setPWM(2, 0, angleToPulse(108));delay(200);
board3.setPWM(3, 0, angleToPulse(113));delay(200);
}
else if (number == 2) {
board2.setPWM(13, 0, angleToPulse(69));delay(200);
board2.setPWM(12, 0, angleToPulse(78));delay(200);
board2.setPWM(11, 0, angleToPulse(80));delay(200);
board2.setPWM(10, 0, angleToPulse(53));delay(200);
board2.setPWM(9, 0, angleToPulse(120));delay(200);
board2.setPWM(8, 0, angleToPulse(81));delay(200);
board2.setPWM(14, 0, angleToPulse(7));delay(200);
board3.setPWM(0, 0, angleToPulse(21));delay(200);
board3.setPWM(3, 0, angleToPulse(13));delay(200);
board3.setPWM(2, 0, angleToPulse(7));delay(200);
}
else if (number == 3) {
board2.setPWM(10, 0, angleToPulse(87));delay(200);
board2.setPWM(9, 0, angleToPulse(85));delay(200);
board3.setPWM(3, 0, angleToPulse(113));delay(200);
}
else if (number == 4) {
board2.setPWM(10, 0, angleToPulse(52));delay(200);
board3.setPWM(3, 0, angleToPulse(74));delay(200);
board2.setPWM(13, 0, angleToPulse(36));delay(200);
board2.setPWM(12, 0, angleToPulse(40));delay(200);
board2.setPWM(11, 0, angleToPulse(114));delay(200);
board2.setPWM(9, 0, angleToPulse(32));delay(200);
board2.setPWM(8, 0, angleToPulse(24));delay(200);
board2.setPWM(14, 0, angleToPulse(41));delay(200);
board3.setPWM(0, 0, angleToPulse(56));delay(200);
board3.setPWM(2, 0, angleToPulse(108));delay(200);
board3.setPWM(3, 0, angleToPulse(113));delay(200);
board2.setPWM(10, 0, angleToPulse(90));delay(200);
}
else if (number == 5) {
board3.setPWM(3, 0, angleToPulse(74));delay(200);
board2.setPWM(13, 0, angleToPulse(72));delay(200);
board2.setPWM(9, 0, angleToPulse(89));delay(200);
board2.setPWM(8, 0, angleToPulse(81));delay(200);
board2.setPWM(15, 0, angleToPulse(2));delay(200);
board2.setPWM(14, 0, angleToPulse(7));delay(200);
board3.setPWM(1, 0, angleToPulse(44));delay(200);
board3.setPWM(0, 0, angleToPulse(21));delay(200);
board3.setPWM(2, 0, angleToPulse(7));delay(200);
board3.setPWM(3, 0, angleToPulse(113));delay(200);
}
}

////////////////////////////////////////////////////////////{{{   DIGIT 4   }}}}
void digit4(int number){
if (number == 00) {
board3.setPWM(6, 0, angleToPulse(39));delay(200);
board3.setPWM(5, 0, angleToPulse(101));delay(200);
}
else if (number == 1) {
board3.setPWM(11, 0, angleToPulse(40));delay(200);
board3.setPWM(9, 0, angleToPulse(23));delay(200);
board3.setPWM(8, 0, angleToPulse(17));delay(200);
board3.setPWM(7, 0, angleToPulse(33));delay(200);
board3.setPWM(6, 0, angleToPulse(24));delay(200);
board3.setPWM(5, 0, angleToPulse(18));delay(200);
board3.setPWM(4, 0, angleToPulse(18));delay(200);
board3.setPWM(14, 0, angleToPulse(42));delay(200);
board3.setPWM(10, 0, angleToPulse(102));delay(200);
board3.setPWM(11, 0, angleToPulse(101));delay(200);
}
else if (number == 2) {
board3.setPWM(9, 0, angleToPulse(75));delay(200);
board3.setPWM(8, 0, angleToPulse(65));delay(200);
board3.setPWM(7, 0, angleToPulse(88));delay(200);
board3.setPWM(6, 0, angleToPulse(40));delay(200);
board3.setPWM(5, 0, angleToPulse(100));delay(200);
board3.setPWM(4, 0, angleToPulse(66));delay(200);
board3.setPWM(14, 0, angleToPulse(7));delay(200);
board3.setPWM(12, 0, angleToPulse(20));delay(200);
board3.setPWM(11, 0, angleToPulse(4));delay(200);
board3.setPWM(10, 0, angleToPulse(8));delay(200);
}
else if (number == 3) {
board3.setPWM(6, 0, angleToPulse(75));delay(200);
board3.setPWM(5, 0, angleToPulse(61));delay(200);
board3.setPWM(11, 0, angleToPulse(103));delay(200);
}
else if (number == 4) {
board3.setPWM(11, 0, angleToPulse(40));delay(200);
board3.setPWM(9, 0, angleToPulse(40));delay(200);
board3.setPWM(8, 0, angleToPulse(32));delay(200);
board3.setPWM(7, 0, angleToPulse(119));delay(200);
board3.setPWM(5, 0, angleToPulse(18));delay(200);
board3.setPWM(4, 0, angleToPulse(18));delay(200);
board3.setPWM(14, 0, angleToPulse(42));delay(200);
board3.setPWM(12, 0, angleToPulse(50));delay(200);
board3.setPWM(10, 0, angleToPulse(102));delay(200);
board3.setPWM(11, 0, angleToPulse(101));delay(200);
}
else if (number == 5) {
board3.setPWM(9, 0, angleToPulse(76));delay(200);
board3.setPWM(5, 0, angleToPulse(65));delay(200);
board3.setPWM(4, 0, angleToPulse(66));delay(200);
board3.setPWM(15, 0, angleToPulse(10));delay(200);
board3.setPWM(14, 0, angleToPulse(8));delay(200);
board3.setPWM(13, 0, angleToPulse(50));delay(200);
board3.setPWM(12, 0, angleToPulse(18));delay(200);
board3.setPWM(10, 0, angleToPulse(7));delay(200);
}
else if (number == 6) {
board3.setPWM(6, 0, angleToPulse(40));delay(200);
board3.setPWM(5, 0, angleToPulse(100));delay(200);
}
else if (number == 7) {
board3.setPWM(11, 0, angleToPulse(40));delay(200);
board3.setPWM(8, 0, angleToPulse(65));delay(200);
board3.setPWM(7, 0, angleToPulse(33));delay(200);
board3.setPWM(6, 0, angleToPulse(23));delay(200);
board3.setPWM(5, 0, angleToPulse(18));delay(200);
board3.setPWM(4, 0, angleToPulse(18));delay(200);
board3.setPWM(15, 0, angleToPulse(48));delay(200);
board3.setPWM(13, 0, angleToPulse(119));delay(200);
board3.setPWM(12, 0, angleToPulse(50));delay(200);
board3.setPWM(10, 0, angleToPulse(102));delay(200);
board3.setPWM(11, 0, angleToPulse(101));delay(200);
}
else if (number == 8) {
board3.setPWM(8, 0, angleToPulse(32));delay(200);
board3.setPWM(7, 0, angleToPulse(118));delay(200);
board3.setPWM(6, 0, angleToPulse(75));delay(200);
board3.setPWM(5, 0, angleToPulse(100));delay(200);
board3.setPWM(4, 0, angleToPulse(66));delay(200);
board3.setPWM(12, 0, angleToPulse(20));delay(200);
board3.setPWM(10, 0, angleToPulse(7));delay(200);
}
else if (number == 9) {
board3.setPWM(6, 0, angleToPulse(76));delay(200);
board3.setPWM(5, 0, angleToPulse(61));delay(200);
board3.setPWM(12, 0, angleToPulse(48));delay(200);
}
}
